//
//  Exception.swift
//
//  Created by Jakob on 14.4.17.
//  Copyright © 2017 Jakob. All rights reserved.
//

import Cocoa

open class Exception: Error
{
	//MARK: Public Static

	public static func alertUser(_ err: String){
		dialog(err, type: .critical)
	}

//	static func warnUser(_ err: String){
//		dialog(err, type: .warning)
//	}
	
//	public static func createFromErrno(msg: String) -> Exception
//	{
//		return Exception(msg: msg, inf: "errno: \(errno), \"\(String(cString: strerror(errno)))\"")
//	}

	public static func alertUser(_ err: Error){
		alertUser(err as NSError)
	}

	/**
	- Parameters:
		- err: original error
		- inf: additional error information
	*/
	public static func alertUser(_ err: Error, inf: String){
		DispatchQueue.main.async {
			let alert = Exception.alert(err)

			if !inf.isEmpty {
				let alertInf = alert.informativeText

				alert.informativeText = alertInf.isEmpty ? inf : inf + "\n" + alertInf
			}
			alert.runModal()
		}
	}

	public static func alertUser(_ err: NSError){
		DispatchQueue.main.async {
			alert(err).runModal()
		}
	}
	
	public static func handle(_ error: Error) {
		if let exc = error as? Exception {
			exc.display()
		} else {
			alertUser(error)
		}
	}

	public static func handle(_ error: Error, _ msg: String)
	{
		if let exc = error as? Exception {
			exc.display()
		} else {
			alertUser(error, inf: msg)
		}
	}

	//MARK: Public Instance Members

	public var message: String
	public var information: String?
	public var cause: Error?

	public init(msg: String){
		message = msg
	}

	public init(msg: String, inf: String?){
		message = msg
		information = inf
	}

	public init(msg: String, cause: Error?){
		message = msg
		self.cause = cause
	}

	public func display() {
		DispatchQueue.main.async {
			let alert = NSAlert()

			alert.messageText = self.message

			if self.information != nil {
				alert.informativeText = self.information!
			}
			alert.runModal()
		}
	}

	//MARK: Private

	private static func dialog(_ err: String, type: NSAlert.Style)
	{
		DispatchQueue.main.async {
			let alert = NSAlert()

			alert.alertStyle = type
			alert.messageText = err
			alert.runModal()
		}
	}

	private static func alert(_ err: Error) -> NSAlert
	{
		return alert(err as NSError)
	}

	private static func alert(_ err: NSError) -> NSAlert
	{
		let alert = NSAlert(error: err)
		let userInf = err.userInfo

		if !userInf.isEmpty {
			var inf = alert.informativeText
			var isFirst = true

			if !inf.isEmpty {
				inf += "\n"
			}

			for (key, value) in userInf {
				if isFirst {
					isFirst = false
				} else {
					inf  += "\n"
				}
				inf += "\(key): \(value)"
			}
			alert.informativeText = inf
		}
		return alert
	}
}
