//
//  OsUtil.swift
//  Pix
//
//  Created by Jakob on 27.1.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

import Cocoa

/**
Access to macOS functions.
*/
public class OsUtil
{
	public static func copyToClipboard(_ str: String)
	{
		let pb = NSPasteboard.general

		pb.clearContents()
		pb.writeObjects([str as NSString])
	}
	
	public static func reveal(_ url: URL){
//		<a href="https://feedbackassistant.apple.com/feedback/8497048">NSWorkspace.activateFileViewerSelecting does not select item in &quot;as Columns&quot; view</a>
		reveal([url])
		// Does not help either:
//		NSWorkspace.shared.selectFile(url.path, inFileViewerRootedAtPath: url.deletingLastPathComponent().path)
	}
	
	public static var stringFromClipboard: String?
	{
		NSPasteboard.general.string(forType: .string)
	}

	public static func reveal(_ urls: [URL]){
		NSWorkspace.shared.activateFileViewerSelecting(urls)
	}
	
	/**
	Opens the Finder's info windows for `items` .
	*/
	public static func openInfoWindow(_ item: URL) {
		let pb = NSPasteboard.withUniqueName()

		pb.writeObjects([item.path as NSString])
		NSPerformService("Finder/Show Info", pb)
	}

	/**
	Opens the Finder's info windows for `items` .
	*/
	public static func openInfoWindow(_ items: [URL]) {
		let pb = NSPasteboard.withUniqueName()
		var ns = [NSString]()

		for item in items {
			ns.append(item.path as NSString)
		}
		pb.writeObjects(ns)
		NSPerformService("Finder/Show Info", pb)
	}

	/**
	Opens the `item` with the default application for its type, e.g. with Finder if it is a folder.
	*/
	public static func open(_ item: URL){
		NSWorkspace.shared.open(item)
	}

	public static func notifyUser(title: String, inf: String? = nil)
	{
		let notification = NSUserNotification()

		notification.title = title

		notification.informativeText = inf
		notification.soundName = NSUserNotificationDefaultSoundName
		NSUserNotificationCenter.default.deliver(notification)
	}
}
