//
//  QuarantineHandler.swift
//  
//
//  Created by Jakob on 05.12.20.
//

import Foundation

class QuarantineHandler
{
	static func deleteQuarantineAttr(_ url: URL) throws
	{
		guard hasQuarantineAttr(url) else {
			return
		}
		
		do {
			try deleteQuarantineAttrWithoutAuth(url)
		} catch  {
			if (error as NSError).code == EACCES // 13
			{
				try deleteQuarantineAttrWithAuth(url)
			} else {
				throw error
			}
		}
	}

	private static let QUARANTINE_ATTR = "com.apple.quarantine"
	private static let XATTR = URL(fileURLWithPath: "/usr/bin/xattr")

	private static func hasQuarantineAttr(_ url: URL) -> Bool
	{
		url.withUnsafeFileSystemRepresentation { fileSystemPath in
			return getxattr(fileSystemPath, QUARANTINE_ATTR, nil, 0, 0, XATTR_NOFOLLOW) >= 0
		}
	}
	
	//TBD Couldn't we just check errno after calling this method, instead of raising an NSError?
	private static func deleteQuarantineAttrWithoutAuth(_ url: URL) throws
	{
		try
			url.withUnsafeFileSystemRepresentation { fileSystemPath in
				let result = removexattr(fileSystemPath, QUARANTINE_ATTR, XATTR_NOFOLLOW)

				if result < 0 {
					throw
						NSError(
							domain: NSPOSIXErrorDomain,
							code: Int(errno),
							userInfo: [NSLocalizedDescriptionKey: String(cString: strerror(errno))]
						)
				}
			}
	}

	/**
	- returns: termination status; 0 is OK
	- throws: `Error`
	*/
	private static func deleteQuarantineRecursively(dir: URL, path: String) throws -> Int32
	{
		let xattr = Process()
	
		xattr.executableURL = XATTR
		xattr.arguments = ["-d", "-r", "com.apple.quarantine", path]
		xattr.currentDirectoryURL = dir

		try xattr.run()
		xattr.waitUntilExit()
		
		return xattr.terminationStatus
	}
	
	private static func deleteQuarantineAttrWithAuth(_ url: URL) throws
	{
		throw Exception(msg: "not implemented")
	}
}
