//
//  WeakArray.swift
//
//
//  Created by Jakob Spies on 06.10.23.
//

import Foundation

/**
 An array of weak references.
 */
public class WeakArray // <T: AnyObject> does not work if T is a protocol
{
	private var content: [Weak<AnyObject>]
	
	public init() {
		content = [Weak<AnyObject>]()
	}
	
	public func add(_ t: AnyObject) {
		content.append(Weak(t))
	}
	
	/**
	 Executes `action` on all contained elements, purges nil references.
	 */
	public func forEach(action: (AnyObject) -> Void) {
		purge()
		content.forEach() { w in
			if let t = w.ref {
				action(t)
			}
		}
	}
	
	/// Removes `nil` references.
	public func purge() {
		content = content.filter() { $0.ref != nil }
	}
}

class Weak<T: AnyObject>
{
	weak var ref: T?
	
	init(_ t: T) {
		ref = t
	}
}
