//
//  DataDownloadTask.swift
//  Filmbeschaffer
//
//  Created by Jakob on 27.10.20.
//  Copyright © 2020 Jakob. All rights reserved.
//

import Foundation

/**
Wrapper for `URLSessionDataTask` taking care of limiting concurrent downloads.
*/
class DataDownloadTask: DownloadTask
{
	override var sessionTask: URLSessionTask
	{
		task
	}

	/**
	analogous to `URLSession.shared.dataTask(with url: URL,
	completionHandler: @escaping (Data?, URLResponse?, Error?) -> Void) -> URLSessionDataTask`
	*/
	convenience init(
		_ url: URL,
		completionHandler: @escaping (Data?, URLResponse?, Error?) -> Void
	){
		self.init(
			URLSession.shared.dataTask(with: url) {
				DownloadTask.semaphore.release()
				completionHandler($0, $1, $2)
			}
		)
	}
	
		
	private let task: URLSessionDataTask

	private init(_ sessionTask: URLSessionDataTask) {
		task = sessionTask
	}
}
