//
//  DownloadAction.swift
//  Filmbeschaffer
//
//  Created by Jakob on 27.10.20.
//  Copyright © 2020 Jakob. All rights reserved.
//

import Foundation

/**
Wrapper for `URLSessionDownloadTask` taking care of limiting concurrent downloads.
*/
class FileDownloadTask: DownloadTask
{
	override var sessionTask: URLSessionTask
	{
		task
	}

	convenience init(urlSession: URLSession, url: URL)
	{
		self.init(urlSession.downloadTask(with: url))
	}
	
	convenience init(urlSession: URLSession, resumeData: Data)
	{
		self.init(urlSession.downloadTask(withResumeData: resumeData))
	}

	/**
	Cancels the wrapped `URLSessionDownloadTask` and releases the connection.
	- parameter byProducingResumeData: what to do with the resume data
	*/
	func cancel(byProducingResumeData: @escaping (Data?) -> Void)
	{
		task.cancel(byProducingResumeData: byProducingResumeData)
		DownloadTask.semaphore.release()
	}
	
	private let task: URLSessionDownloadTask
	
	private init(_ sessionTask: URLSessionDownloadTask) {
		task = sessionTask
	}
}
