//
//  Download.swift
//  Filmbeschaffer
//
//  Created by Jakob on 8.12.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

import Foundation

protocol Download
{
	var url: URL
	{ get }

	var target: URL
	{ get }

	var status: DownloadState
	{ get }

	/// bytes
	var downloaded: Int64
	{ get }

	/// bytes
	var totalSize: Int64
	{ get }

	/// bytes/s
	var speed: Double
	{ get }

	/// Says whether this `Download` is in a terminal state.
	var isFinished: Bool
	{ get }

	/**
	Says whether we have changed the state, but nobody in the outside world can know the new state,
	because nobody has inspected `status` since.
	*/
	var isNewState: Bool
	{ get }

	var isResumable: Bool
	{ get }

	/// Equivalent to isCancellable.
	var isActive: Bool
	{ get }

	var errorMessage: String?
	{ get }

	/// Suspends the download.
	func cancel()

	/// Starts or resumes the download.
	func start()

	/// Cleans up used resources. Download cannot be resumed afterwards.
	func destroy()
	
	/// Opens the downloaded video, if possible.
	func openVideo()
}


enum DownloadState
{
	case

	/// initial
	waiting,

	/// initial
	preparing,

	downloading, cancelling,

	//TBD check whether this is equivalent to cancelled
	/// resumable if `isResumable`
	error,


	/// resumable
	cancelled,

	/// final
	finished
}

