//
//  Downloader.swift
//  M3U8
//
//  Created by Jakob on 17.5.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

import Foundation
//import os.log

/// Basic download functionality.
class Downloader
{
	//MARK: Static
	
	static let FILES 		= FileManager.default
	static let FILE_ATTR = [FileAttributeKey.posixPermissions : 0o644]

	
	//MARK: Public Instance

	var theError: Error?
	{
		didSet {
			if isResumable {
				if let err = theError {
					resumeData = (err as NSError).userInfo[NSURLSessionDownloadTaskResumeData] as? Data
				}
			}
		}
	}

	var data: Data?
	var myUrl: URL!
	var resumeData: Data?

	var haveError: Bool
	{
		return theError != nil
	}

	init(_ withResume: Bool = false){
		isResumable = withResume
	}

	/**
	 After return of this function, the raw data is available as `data!`.
	- throws: `DownloadError`, `Error`
	*/
	func getUtf8TextPage(_ url: URL, headers: [String:String]? = nil) throws -> String
	{
//		os_log("getting page %@", log: OSLog.default, type: .debug, url as CVarArg)
		try getFrom(url, headers: headers)
	
		let x = try dataAsString()
//		os_log("got page %@", log: OSLog.default, type: .debug, url as CVarArg)
		return x
	}
	
	//MARK: Private Instance

	private let isResumable: Bool

	/// Serves for informing clients about the end of a download.
	private var join: NSCondition!

	private func initJoin(){
		join = NSCondition()
		join.name = "wait for response"
		join.lock()
	}

	private func run(_ task: URLSessionDataTask) throws
	{
		data = nil
		initJoin()
		task.resume()

		let end = Date(timeIntervalSinceNow: 30)

		while !haveError && data == nil {
			join.wait(until: end)
		}
		join.unlock()
		try throwOnError()
	}
	private func throwOnError() throws
	{
		if haveError {
			throw theError!
		}
	}
	
	private func signalFinished() {
		join.lock()
		join.signal()
		join.unlock()
	}

	/**
	- throws: `DownloadError`
	*/
	private func dataAsString() throws -> String
	{
		guard let page = String(data: data!, encoding: .utf8) else {
			throw DownloadError("cannot read text from " + String(describing: myUrl))
		}
		return page
	}

	private func completionHandler(data: Data?, response: URLResponse?, error: Error?){
		if let error = error {
			theError = error
		}else if data == nil {
			theError = DownloadError("got no error and no data from " + String(describing: myUrl))
		}else{
			self.data = data
		}
		signalFinished()
	}

	/**
	- throws: `DownloadError`, `Error`
	*/
	private func getFrom(_ url: URL, headers: [String:String]? = nil) throws
	{
		myUrl = url
		
		var urlRequest = URLRequest(url: myUrl)

		if let theHeaders = headers {
			for (key, value) in theHeaders {
				urlRequest.setValue(value, forHTTPHeaderField: key)
			}
		}
		try run(URLSession.shared.dataTask(with: urlRequest, completionHandler: completionHandler))
	}
}
