//
//  Download.swift
//  Filmbeschaffer
//
//  Created by Jakob on 20.5.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

//import os.log

import Cocoa
import AppUtilLib

class FileDownload: DownloadSkeleton
{
	//MARK: Public Instance

	override var totalSize: Int64
	{
		return fileDownloadTotalSize
	}

	override var speed: Double
	{
		let now = Date()
		
		let nBytes = fileDownloadedBytes
		// because the value could be already greater when we reach the end of the method
		
		var event: Event?
		var i = 0
		var time = 0.0
		var diff = 0.0
		var previousEvent: Event?

		internalSynchronizationQueue.sync
		{
			let n = events.count // we know n > 0
			
			while i < n
			{
				event = events[i]
				diff = now.timeIntervalSince(event!.time) // > 0
				
				if diff < minPast {
					break
				}
				time = diff
				previousEvent = event
				i += 1
			}
		}
		// diff > 0
		
		if i == 0 // no event old enough
		{
			time = diff // using the last Event we have inspected
		}
		else // time comes from the Event before the last one we have inspected
		{
			event = previousEvent
		}

		return Double(nBytes - event!.bytes) / time
	}

	/// runs in main thread, so ought to be quick
	override init(url u: URL, target to: URL) throws
	{
		guard let host = u.host else {
			throw DownloadError("Page URL \(u) has no host.")
		}

		if host.range(of: ".zdf") != nil {
			urlFinder = ZdfInfoUrlFinder()
		} else {
			throw DownloadError("Page URL \(u) does not belong to ZDF or arte.tv.")
		}

		try super.init(url: u, target: to)
	}

	override func start()
	{
		downloadQueue.async {
			self.download()
		}
	}

	override func cancel(){
		internalSynchronizationQueue.sync { // prevent other threads from changing downloadTask
			guard let dt = fileDownloadTask else {
				return
			}
			state = .cancelling

			dt.cancel() { data in
				self.downloader.resumeData = data
				self.signalFinished(.cancelled)
			}
		}
	}

	override func openVideo() {
		OsUtil.open(target)
	}

	//MARK: Private

	private let urlFinder: UrlFinder
	private var videoUrl: URL?

	/**
	Starts the download, waits for a connection, but does not wait for the
	download to finish.
	Runs in `downloadQueue`.
	*/
	private func download()
	{
		do {
			if videoUrl == nil {
				videoUrl = try urlFinder.findVideoUrl(downloadUrl)
			}
			
			download(videoUrl)
		} catch let e as DownloadError {
			state = .error
			
			DispatchQueue.main.async {
				let alert = NSAlert(error: e)
				alert.messageText = alert.messageText + "\n" + e.message

				alert.runModal()
			}
		} catch {
			state = .error

			DispatchQueue.main.async {
				NSAlert(error: error).runModal()
			}
		}
	}
}
