//  Preferences.swift
//  Created by Jakob on 19.4.17.
//  Copyright © 2017 Jakob. All rights reserved.

import Cocoa

/// Singleton.
class Preferences
{
	// MARK: Public

	static let WWW_PATH = "fb"
	static let APP_NAME = "Filmbeschaffer"

	static var instance: Preferences
	{
		get {
			let object = Preferences.self // arbitrary object

			// poor substitute for 'synchronized'
			objc_sync_enter(object)
			defer { objc_sync_exit(object) }

			if theInstance == nil {
				theInstance = Preferences()
			}
			return theInstance!
		}
	}
	
	var speedCalculationTimeSpan: TimeInterval
	{
		get {
			var result = UserDefaults.standard.double(forKey: Preferences.KEY_SPEED_CALC_TIME_SPAN)

			if result < 1 {
				result = 12
			}
			return result
		}

		set(value){
			UserDefaults.standard.set(value, forKey: Preferences.KEY_SPEED_CALC_TIME_SPAN)
		}
	}

	var isCheckVersion: Bool
	{
		get {
			// Note the negation! The purpose is that, if the setting does not exist,
			// then we automatically return true.
			return !UserDefaults.standard.bool(forKey: Preferences.KEY_CHECK_VERSION)
		}

		set(i) {
			// Note the negation, cf. above!
			UserDefaults.standard.set(!i, forKey: Preferences.KEY_CHECK_VERSION)
		}
	}

	//TBD make configurable
	let maxConnections = 5
	
	// MARK: Private Static

	private static let KEY_SPEED_CALC_TIME_SPAN	= "speed calculation time span"
	private static let KEY_CHECK_VERSION			= "check version"

	private static var theInstance: Preferences?
}
