//
//  ZdfInfoUrlFinder.swift
//  Filmbeschaffer
//
//  Created by Jakob on 30.5.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

import Foundation
import os.log

class ZdfInfoUrlFinder: Downloader, UrlFinder
{
	//MARK: Public

	func findVideoUrl(_ baseUrl: URL) throws -> URL
	{
		var string = try getUtf8TextPage(baseUrl)
		var json = try HtmlUtil.findJson("data-zdfplayer-jsb", in: string, endJson: "}'", offset: 0, desc: baseUrl)
		let authToken = try HtmlUtil.stringAttribute("apiToken", in: json)
		var newUrlComp = URLComponents()

		auth = "Bearer " + authToken
		string = try HtmlUtil.stringAttribute("content", in: json)
		myUrl = try HtmlUtil.toUrl(string)

		guard let urlComponents = URLComponents(url: myUrl, resolvingAgainstBaseURL: false) else {
			throw DownloadError(String(describing: myUrl) + " cannot be decomposed")
		}

		newUrlComp.scheme = urlComponents.scheme
		newUrlComp.host = urlComponents.host
		newUrlComp.port = urlComponents.port

		json = try loadJson()
		json = try HtmlUtil.attribute("mainVideoContent", in: json)

		let dic1 = try HtmlUtil.toDictionary(json, desc: json)
		var foundObject: Any?

		for (key, value) in dic1 {
			if key.hasPrefix(ZdfInfoUrlFinder.ATTR_PREFIX){
				foundObject = value
				break
			}
		}

		guard let dic2 = foundObject as? [String:Any] else {
			throw DownloadError("\(dic1) does not contain a dictionary attribute the key of which starts with \(ZdfInfoUrlFinder.ATTR_PREFIX)")
		}

		var path: String?

		for value in dic2.values {
			if let p = value as? String {
				if let rg = p.range(of: ZdfInfoUrlFinder.PLAYER_ID) {
					path = p
					path!.replaceSubrange(rg, with: "ngplayer_2_3")
					break
				}
			}
		}

		path = "/content/documents/vod-ard-tod-in-der-badewanne---unter-verdacht-100.json?profile=tmd"
		guard let newPath = path else {
			throw DownloadError("\(dic2) does not contain a value containing \"\(ZdfInfoUrlFinder.PLAYER_ID)\"")
		}

		newUrlComp.path = newPath

		guard let newUrl = newUrlComp.url else {
			throw DownloadError("cannot build URL from \(newUrlComp)")
		}
		myUrl = newUrl
		json = try loadJson()
		foundObject = nil

		var arr1 = try HtmlUtil.arrayAttribute(ZdfInfoUrlFinder.PRIORITY_LIST, in: json, desc: "attribute \(ZdfInfoUrlFinder.PRIORITY_LIST)")

		loop:
		for x in arr1 {
			json = try HtmlUtil.firstObjOfArrayAttribute(ZdfInfoUrlFinder.FORMITAETEN, in: x, desc: ZdfInfoUrlFinder.ATTR_FORMITAETEN)

			let arr3 = try HtmlUtil.arrayAttribute("facets", in: json)

			for y in arr3 {
				if y as? String == ZdfInfoUrlFinder.PROGRESSIVE {
					foundObject = json
					break loop
				}
			}
		}

		guard let formitaet = foundObject else {
			throw DownloadError("could not find \"Formität\" with \"\(ZdfInfoUrlFinder.PROGRESSIVE)\" facet")
		}

		arr1 = try HtmlUtil.arrayAttribute(ZdfInfoUrlFinder.QUALITIES, in: formitaet)
		foundObject = nil

		for x in arr1 {
			string = try HtmlUtil.stringAttribute(ZdfInfoUrlFinder.QUALITY, in: x)

			if string == ZdfInfoUrlFinder.VERYHIGH {
				foundObject = x
				break
			}
		}

		guard let obj1 = foundObject else {
			throw DownloadError("could not find object with \"\(ZdfInfoUrlFinder.VERYHIGH)\" \"\(ZdfInfoUrlFinder.QUALITY)\" in \(arr1)")
		}
		json = try HtmlUtil.attribute("audio", in: obj1)
		json = try HtmlUtil.firstObjOfArrayAttribute("tracks", in: json, desc: json)
		string = try HtmlUtil.stringAttribute("uri", in: json)
		return try HtmlUtil.toUrl(string)
	}

	//MARK: Private

	private static let ATTR_PREFIX = "http"
	private static let PRIORITY_LIST = "priorityList"
	private static let FORMITAETEN = "formitaeten"
	private static let ATTR_FORMITAETEN = "attribute " + FORMITAETEN
	private static let PROGRESSIVE = "progressive"
	private static let VERYHIGH = "veryhigh"
	private static let QUALITY = "quality"
	private static let QUALITIES = "qualities"
	private static let PLAYER_ID = "{playerId}"

	private var auth: String?

	private func loadJson() throws -> Any
	{
		os_log("downloading JSON %@", log: OSLog.default, type: .debug, myUrl as CVarArg)

		let json = try getUtf8TextPage(myUrl, headers: auth == nil ? nil : ["Api-Auth" : auth!])
print(json)
		return try HtmlUtil.deserialize(json)
	}
}
