//
//  DownloadWindow.swift
//  Filmbeschaffer
//
//  Created by Jakob on 20.5.18.
//  Copyright © 2018 Jakob. All rights reserved.
//

import Cocoa

/**
This object does not get garbage-collected because of the mutual dependency with the `urlField`.
This is o.k.
*/
class DownloadWindowController: NSWindowController, NSTextFieldDelegate
{
	//MARK: Public
	
	static func create() -> DownloadWindowController
	{
		return DownloadWindowController(windowNibName: "DownloadWindow")
	}

	@IBOutlet var urlLabel: NSTextField!
	@IBOutlet var urlField: NSTextField!
	@IBOutlet var startButton: NSButton!
	var listener: DownloadEventListener?
	

	@IBAction
	func edited(_ sender: NSTextField) {
		edited()
	}
	
	/**
	 Called by start button.
	 */
	@IBAction func chooseFile(_ sender: Any)
	{
		guard let internetUrl = theUrl else {
			fatalError("chooseFile can only be called if theUrl != nil!")
		}
		
		window?.close()
		
		guard let host = internetUrl.host else {
			let alert = NSAlert()
			
			alert.alertStyle = .critical
			alert.messageText = "Page URL \(internetUrl) has no host."
			alert.runModal()
			return
		}

		let isArte = host.range(of: ".arte.") != nil
		
		if isArte {
			isStream = true
		}
		
		let panel: NSSavePanel
		
		if isStream
		{
			panel = NSOpenPanel()
			
			let op = (panel as! NSOpenPanel)
			
			op.allowsMultipleSelection = false
			op.canChooseDirectories = true
			op.canChooseFiles = false
		}
		else
		{
			panel = NSSavePanel()
			panel.nameFieldStringValue = proposedFileName
		}
		
		panel.title = "Download Video"
		panel.message = "Please choose a download destination!"
		panel.canCreateDirectories = true
		
		if panel.runModal() == .OK {
			let fileUrl = panel.url!
	
			do {
				let download =
					isStream ?
							try StreamDownload(url: internetUrl, target: fileUrl, isArte: isArte)
						:
							try FileDownload(url: internetUrl, target: fileUrl)
				
				listener?.process(download)
			} catch let e as DownloadError {
				let alert = NSAlert(error: e)
				alert.messageText = alert.messageText + "\n" + e.message
				alert.runModal()
			} catch {
				let alert = NSAlert(error: error)
				alert.runModal()
			}
		}
	}

	func controlTextDidChange(_ obj: Notification){
		edited()
	}

	func show(_ stream: Bool = false)
	{
		guard let window = window else {
			NSSound.beep()
			return
		}

		isStream = stream
		
		if isStream {
			urlLabel.stringValue = "Stream URL"
		} else {
			urlLabel.stringValue = "Page URL"
		}
		window.makeKeyAndOrderFront(self)
	}

	//MARK: Private
	
	private static let HTML = ".html"
	private static let HTML_OFFSET = -HTML.count
	private static let FILE_SUFFIX = ".mp4"

	// Would be `let` if Swift's constructor rules were not so stupid.
	private var isStream = false
	
	private var theUrl: URL?

	private var proposedFileName: String
	{
		var url = urlField.stringValue

		if url.hasSuffix(HtmlUtil.URL_COMPONENT_SEPARATOR) {
			url.removeLast()
		} else if url.hasSuffix(".html") {
			let end = url.endIndex
			url.removeSubrange(url.index(end, offsetBy: DownloadWindowController.HTML_OFFSET) ..< end)
		}

		let last = url.components(separatedBy: HtmlUtil.URL_COMPONENT_SEPARATOR).last

		return last == nil ? DownloadWindowController.FILE_SUFFIX : last! + DownloadWindowController.FILE_SUFFIX
	}

	private func edited() {
		theUrl = URL(string: urlField.stringValue)
		startButton.isEnabled = theUrl != nil
	}
}
